# coding: utf-8

"""
    Graphsignal API

    API for uploading and querying spans, errors, metrics, and logs.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class MetricType(str, Enum):
    """
    MetricType
    """

    """
    allowed enum values
    """
    GAUGE = 'gauge'
    COUNTER = 'counter'
    SUMMARY = 'summary'
    HISTOGRAM = 'histogram'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of MetricType from a JSON string"""
        return cls(json.loads(json_str))


